
//*****************************************************************************
//
//  Title          :  SLL Hyperbus Controller PLL
//  Project        :  N/A
//
//  File           :  sll_ca_hbc_t001_syncpll.v
//  Author         :  Mark Bonnici
//
//  Email          :  mark.bonnici@synaptic-labs.com
//  Revision date  :  Feb/15/2017
//
//  Description    : SLL Hyperbus Controller PLL generation
//
//
//  Known issues   :  None
//
//  References     :
//
//*********************************************************************************************************************************
//  Revision     Date         Author         Comments
//
//  Version 2.11 28/Apr/2017   Mark     changed parameters for altpll module
//*********************************************************************************************************************************
//                   75/75      100/100  125/125     150/75      150/100    150/125  165/75      160/100    165/115   
// clk0_divide_by     = 2,       1,        2,        = 1,        = 1,      = 1,        = 10,      5,          10,     
// clk0_duty_cycle    = 50,      50,       50,       = 50,       = 50,     = 50,       = 50,      50,         50,     
// clk0_multiply_by   = 3,       2,        5,        = 3,        = 3,      = 3,        = 33,      16,         33,     
// clk0_phase_shift   = "0",     "0",      "0",      = "0",      = "0",    = "0",      = "0",     "0",        "0",    
// clk1_divide_by     = 2,       1,        2,        = 1,        = 1,      = 1,        = 10,      5,          10,     
// clk1_duty_cycle    = 50,      50,       50,       = 50,       = 50,     = 50,       = 50,      50,         50,     
// clk1_multiply_by   = 3,       2,        5,        = 3,        = 3,      = 3,        = 33,      16,         33,     
// clk1_phase_shift   = "3333",  "2500",   "2000",   = "1667",   = "1667", = "1667",   = "1515",  "1563",     "1515", 
// clk2_divide_by     = 2,       1,        2,        = 1,        = 1,      = 1,        = 10,      5,          10,     
// clk2_duty_cycle    = 50,      50,       50,       = 50,       = 50,     = 50,       = 50,      50,         50,     
// clk2_multiply_by   = 3,       2,        5,        = 3,        = 3,      = 3,        = 33,      16,         33,     
// clk2_phase_shift   = "6667",  "5000",   "4000",   = "3333",   = "3333", = "3333",   = "3030",  "3125",     "3030", 
// clk3_divide_by     = 2,       1,        2,        = 1,        = 1,      = 1,        = 10,      5,          10,     
// clk3_duty_cycle    = 100,     100,      100,      = 100,      = 100,    = 100,      = 100,     100,        100,    
// clk3_multiply_by   = 3,       2,        5,        = 3,        = 3,      = 3,        = 33,      16,         33,     
// clk3_phase_shift   = "10000", "7500",   "6000",   = "5000",   = "5000", = "5000",   = "4545",  "4688",     "4545", 
// clk4_divide_by     = 2,       1,        2,        = 2,        = 1,      = 2,        = 2,       1,          5,       
// clk4_duty_cycle    = 50,      50,       50,       = 50,       = 50,     = 50,       = 50,      50,         50,     
// clk4_multiply_by   = 3,       2,        5,        = 3,        = 2,      = 5,        = 3,       2,          11,     


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on
module sll_ca_hbc_t001_syncpll (
	areset,
	inclk0,
	c0,
	c1,
	c2,
	c3,
	c4,
	locked
);
 parameter g_input_clk_freq_in_mhz = 50;      
 parameter g_hyperbus_freq_in_mhz  = 100;     //hyperbuus memory frequency
 parameter g_core_freq_in_mhz      = 100;     //Abvalon-MM core frequecy
 parameter g_device_family         = "MAX 10";   

	input	    areset;
	input	    inclk0;
	output	  c0;
	output	  c1;
	output	  c2;
	output	  c3;
	output	  c4;
	output	  locked;
`ifndef ALTERA_RESERVED_QIS
// synopsys translate_off
`endif
	tri0	  areset;
`ifndef ALTERA_RESERVED_QIS
// synopsys translate_on
`endif

//input frequency    - supported frequencies : 50 Mhz  
//hyperbus frequency - supported frequencies : 75 100 125 150
//core frequency     - supported frequencies : 75 100 125 150

//------------------------------------------------------------------
//Compute parameters for the PLL clock generation
//------------------------------------------------------------------
localparam g_input_clk_freq_in_ps = (1000000/g_input_clk_freq_in_mhz);


localparam clk0_divide_by   = (g_hyperbus_freq_in_mhz == 75  || g_hyperbus_freq_in_mhz == 125) ? 2 : 
                              (g_hyperbus_freq_in_mhz == 110 || g_hyperbus_freq_in_mhz == 160) ? 5 :
                              (g_hyperbus_freq_in_mhz == 165) ? 10 : 1;


localparam clk0_multiply_by = (g_hyperbus_freq_in_mhz*clk0_divide_by)/g_input_clk_freq_in_mhz;        


localparam clk4_divide_by   = (g_core_freq_in_mhz == 75  || g_core_freq_in_mhz == 125) ? 2 : 
                              (g_core_freq_in_mhz == 110 || g_core_freq_in_mhz == 160) ? 5 :
                              (g_core_freq_in_mhz == 165) ? 10 : 1;

localparam clk4_multiply_by = (g_core_freq_in_mhz*clk4_divide_by)/g_input_clk_freq_in_mhz;


localparam clk1_shift       = (g_hyperbus_freq_in_mhz==165) ? "1515" :
                              (g_hyperbus_freq_in_mhz==160) ? "1563" :
                              (g_hyperbus_freq_in_mhz==150) ? "1667" :
                              (g_hyperbus_freq_in_mhz==125) ? "2000" :
                              (g_hyperbus_freq_in_mhz==100) ? "2500" : "3333";

localparam clk2_shift       = (g_hyperbus_freq_in_mhz==165) ? "3030" :
                              (g_hyperbus_freq_in_mhz==160) ? "3125" :
                              (g_hyperbus_freq_in_mhz==150) ? "3333" :
                              (g_hyperbus_freq_in_mhz==125) ? "4000" :
                              (g_hyperbus_freq_in_mhz==100) ? "5000" : "6667";
                              
localparam clk3_shift       = (g_hyperbus_freq_in_mhz==165) ? "4545" :
                              (g_hyperbus_freq_in_mhz==160) ? "4688" :
                              (g_hyperbus_freq_in_mhz==150) ? "5000" :
                              (g_hyperbus_freq_in_mhz==125) ? "6000" :
                              (g_hyperbus_freq_in_mhz==100) ? "7500" : "10000";
//synopsys translate_off
initial
begin
  if (g_input_clk_freq_in_mhz!=50) begin
  	 $display("Pll Input Frequency should be set to 50 Mhz");
     $stop;  	 
  end

  if ((g_hyperbus_freq_in_mhz==165) ||
      (g_hyperbus_freq_in_mhz==160) ||
      (g_hyperbus_freq_in_mhz==150) ||
      (g_hyperbus_freq_in_mhz==125) ||
      (g_hyperbus_freq_in_mhz==110) ||
      (g_hyperbus_freq_in_mhz==100) ||
      (g_hyperbus_freq_in_mhz==75) ) begin
  end else begin
  	 $display("hyperbus frequency should be set to 75, 100, 125, 150, 160 or 165 Mhz");
     $stop;  	 
  end

  if ((g_core_freq_in_mhz==150) ||
      (g_core_freq_in_mhz==125) ||
      (g_core_freq_in_mhz==110) ||
      (g_core_freq_in_mhz==100) ||
      (g_core_freq_in_mhz==75) ) begin
  end else begin
  	 $display("Avalon-MM frequency should be set to 75, 100, 125, or 150 Mhz");
     $stop;  	 
  end
	 
end                              
//synopsys translate_on

	wire  locked, c0, c1, c2, c3, c4;

	altpll	#(
	   .bandwidth_type   ("AUTO")
	  ,.clk0_divide_by   (clk0_divide_by     )
	  ,.clk0_duty_cycle  (50    )
	  ,.clk0_multiply_by (clk0_multiply_by   )
	  ,.clk0_phase_shift ("0"   )
	  ,.clk1_divide_by   (clk0_divide_by     )
	  ,.clk1_duty_cycle  (50    )
	  ,.clk1_multiply_by (clk0_multiply_by   )
	  ,.clk1_phase_shift (clk1_shift         )
	  ,.clk2_divide_by   (clk0_divide_by     )
	  ,.clk2_duty_cycle  (50    )
	  ,.clk2_multiply_by (clk0_multiply_by   )
	  ,.clk2_phase_shift (clk2_shift         )
	  ,.clk3_divide_by   (clk0_divide_by     )
	  ,.clk3_duty_cycle  (50    )
	  ,.clk3_multiply_by (clk0_multiply_by   )
	  ,.clk3_phase_shift (clk3_shift         )
	  ,.clk4_divide_by   (clk4_divide_by     )
	  ,.clk4_duty_cycle  (50    )
	  ,.clk4_multiply_by (clk4_multiply_by   )
	  ,.clk4_phase_shift ("0"                )
	  ,.compensate_clock ("CLK0")
	  ,.inclk0_input_frequency (g_input_clk_freq_in_ps)
	  ,.intended_device_family (g_device_family)
	  ,.lpm_hint         ("CBX_MODULE_PREFIX=syncpll")
	  ,.lpm_type         ("altpll"     )
	  ,.operation_mode   ("NORMAL"     )
	  ,.pll_type         ("AUTO"       )
	  ,.port_activeclock ("PORT_UNUSED")
	  ,.port_areset      ("PORT_USED"  )
	  ,.port_clkbad0     ("PORT_UNUSED")
	  ,.port_clkbad1     ("PORT_UNUSED")
	  ,.port_clkloss     ("PORT_UNUSED")
	  ,.port_clkswitch   ("PORT_UNUSED")
	  ,.port_configupdate("PORT_UNUSED")
	  ,.port_fbin        ("PORT_UNUSED")
	  ,.port_inclk0      ("PORT_USED"  )
	  ,.port_inclk1      ("PORT_UNUSED")
	  ,.port_locked      ("PORT_USED"  )
	  ,.port_pfdena      ("PORT_UNUSED")
	  ,.port_phasecounterselect("PORT_UNUSED")
	  ,.port_phasedone   ("PORT_UNUSED")
	  ,.port_phasestep   ("PORT_UNUSED")
	  ,.port_phaseupdown ("PORT_UNUSED")
	  ,.port_pllena      ("PORT_UNUSED")
	  ,.port_scanaclr    ("PORT_UNUSED")
	  ,.port_scanclk     ("PORT_UNUSED")
	  ,.port_scanclkena  ("PORT_UNUSED")
	  ,.port_scandata    ("PORT_UNUSED")
	  ,.port_scandataout ("PORT_UNUSED")
	  ,.port_scandone    ("PORT_UNUSED")
	  ,.port_scanread    ("PORT_UNUSED")
	  ,.port_scanwrite   ("PORT_UNUSED")
	  ,.port_clk0        ("PORT_USED"  )
	  ,.port_clk1        ("PORT_USED"  )
	  ,.port_clk2        ("PORT_USED"  )
	  ,.port_clk3        ("PORT_USED"  )
	  ,.port_clk4        ("PORT_USED"  )
	  ,.port_clk5        ("PORT_UNUSED")
	  ,.port_clkena0     ("PORT_UNUSED")
	  ,.port_clkena1     ("PORT_UNUSED")
	  ,.port_clkena2     ("PORT_UNUSED")
	  ,.port_clkena3     ("PORT_UNUSED")
	  ,.port_clkena4     ("PORT_UNUSED")
	  ,.port_clkena5     ("PORT_UNUSED")
	  ,.port_extclk0     ("PORT_UNUSED")
	  ,.port_extclk1     ("PORT_UNUSED")
	  ,.port_extclk2     ("PORT_UNUSED")
	  ,.port_extclk3     ("PORT_UNUSED")
	  ,.self_reset_on_loss_lock  ("OFF")
	  ,.width_clock      (5)
	) altpll_component (
				.areset       (areset),
				.inclk        ({1'b0, inclk0}),
				.clk          ({c4, c3, c2, c1, c0}),
				.locked       (locked),
				.activeclock  (),
				.clkbad       (),
				.clkena       ({6{1'b1}}),
				.clkloss      (),
				.clkswitch    (1'b0),
				.configupdate (1'b0),
				.enable0      (),
				.enable1      (),
				.extclk       (),
				.extclkena    ({4{1'b1}}),
				.fbin         (1'b1),
				.fbmimicbidir (),
				.fbout        (),
				.fref         (),
				.icdrclk      (),
				.pfdena       (1'b1),
				.phasecounterselect ({4{1'b1}}),
				.phasedone    (),
				.phasestep    (1'b1),
				.phaseupdown  (1'b1),
				.pllena       (1'b1),
				.scanaclr     (1'b0),
				.scanclk      (1'b0),
				.scanclkena   (1'b1),
				.scandata     (1'b0),
				.scandataout  (),
				.scandone     (),
				.scanread     (1'b0),
				.scanwrite    (1'b0),
				.sclkout0     (),
				.sclkout1     (),
				.vcooverrange (),
				.vcounderrange()
	);	


endmodule
